
/* ----------------------------------------------------------------------
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  Purpose: RINEX/RTCM decoder, QC, concatenation (focus on navigation messages)
  Version: $ Rev: $

  2014-10-23 /JD: created

-*/

#include "gset/gcfg_aset.h"

using namespace std;

void catch_signal(int){ cout << "Program interrupted by Ctrl-C [SIGINT,2]\n"; exit(1); }

void usage(const string& app){   
  cout << app  << endl;
  cout << endl << "Usage: "
       << endl
       << endl << "    -h|-help               .. this help                          "
       << endl << "    -V int                 .. version                            "
       << endl << "    -v int                 .. verbosity level                    "
       << endl << "    -x file                .. configuration input file           "
       << endl << "    -l file                .. log output file                    "     
       << endl << "    -X                     .. output default configuration in XML"
       << endl << endl;
  exit(0);
}

/* -----
 * MAIN
-*/
int main(int argc, char *argv[])
{  
  signal(SIGINT, catch_signal);
  string conf;
  t_glog glog;
    glog.mask("/dev/stdout");
    glog.cache_size(1);

  t_gcfg_aset gset;
  gset.app("G-Nut/Aset", "0.9.0", "$Rev:$","(gnss@pecny.cz)", __DATE__, __TIME__);

  for( int i=1; i<argc; i++ ){
    if (  !strcmp(argv[i],"--help") && i  <argc ){ usage(gset.app()); exit(0); }
    else if ( !strcmp(argv[i],"-h") && i  <argc ){ usage(gset.app()); exit(0); }
    else if ( !strcmp(argv[i],"-V") && i  <argc ){ cout << gset.app() << endl; exit(0); }
    else if ( !strcmp(argv[i],"-X") && i  <argc ){ gset.help(); exit(0); }
    else if ( !strcmp(argv[i],"-x") && i+1<argc ){ conf = argv[++i]; }
    else if ( !strcmp(argv[i],"-v") && i+1<argc ){ glog.verb(atoi(argv[++i]));}
    else if ( !strcmp(argv[i],"-l") && i+1<argc ){ glog.mask(argv[++i]); }
  }
  if( conf.empty() ){ if( gset.read_istream(cin) ){ gset.help(); exit(1); }}
  else{               if( gset.read(conf)        ){ gset.help(); exit(1); }}
  gset.glog( &glog );

  glog.comment(0,"main",gset.app());
   
  // data containers
  t_gallnav* gnav   = 0;
  t_gcoder*  gcoder = 0;
  t_gio*     gio    = 0;

  gnav = new t_gallnav; gnav->glog( &glog );
   
  // read inputs
  // -----------
  string path;
  int idx = 0;
  multimap<IFMT,string> inp = gset.inputs_all();
  multimap<IFMT,string>::const_iterator itINP = inp.begin();
  for( size_t i = 0; i < inp.size() && itINP != inp.end(); ++i, ++itINP ){
     
    IFMT   ifmt(itINP->first);
    path = itINP->second;
    string id("ID" + int2str(i));


    if(       ifmt == RINEXN_INP    ){ gcoder = new t_rinexn("",4096);
//  }else if( ifmt == RTCM_INP      ){ gcoder = new t_rtcm("",4096);

                                    // gcoder = new t_rtcm2data("",1024); // <<=====
                                    // gcoder = new t_rtcmdata("",1024);
                                    // gcoder = new t_rtcmprod("",1024); 

    }else{    glog.comment(0,"main","Error: decoder not supported : "+ int2str(ifmt) );
              continue;
    }

    gcoder->clear();
    gcoder->glog(&glog);
    gcoder->gset(&gset);
    gcoder->add_data("OBJ", gnav);

    gio = new t_gfile;    // file considered now only;
    gio->glog(&glog);
    gio->path(path);
    gio->coder(gcoder);

    clock_t tmpclk = clock();
    glog.comment(1,"main","READ: " + gio->path() + " started");
    gio->run_read();
    glog.comment(0,"main","READ: " + gio->path() + " " + clk_string(tmpclk));

    delete gio;
    delete gcoder;
    idx++;
  }

   
  // write output
  // ============
  set<string> outf = gset.oformats();
  string ofmt = "rinexn";
  if( outf.find( ofmt ) != outf.end() ){
    gcoder = new t_rinexn("",4096);
     
    path = gset.outputs( ofmt );
    gcoder = new t_rinexn("3.1",4096);
    gcoder->glog(&glog);
    gcoder->gset(&gset);
    gcoder->add_data("OBJ", gnav);

    gio = new t_gfile;
    gio->glog(&glog);
    gio->path(path);
    gio->coder(gcoder);
          
    clock_t tmpclk = clock();
    glog.comment(1,"main","SAVE: " + gio->path() + " started");
    gio->run_write();
    glog.comment(0,"main","SAVE: " + gio->path() + " " + clk_string(tmpclk));
     
    delete gio;
    delete gcoder;
  }


// control output
// ==============
  t_gtime epoch(t_gtime::GPS);
  double xyz[3];
  double clk;
  string prn("G03");
      
  // FIRST MESSAGE
  epoch = gnav->beg_gnav( prn );
  gnav->pos( prn, epoch, xyz );
  gnav->clk( prn, epoch, &clk );
  ostringstream os;
    os << epoch.str_ymdhms("BEG: ")
       << fixed   << setprecision(3)
       << setw(6) << prn   
       << setw(16)<< xyz[0]
       << setw(16)<< xyz[1]
       << setw(16)<< xyz[2]
       << setprecision(6)
       << setw(16)<< clk*1000000; // msec
  glog.comment(0,"test",os.str()); os.str("");

  // LAST MESSAGE
  epoch = gnav->end_gnav( prn );
  gnav->pos( prn, epoch, xyz );
  gnav->clk( prn, epoch, &clk );
    os << epoch.str_ymdhms("END: ")
       << fixed   << setprecision(3)
       << setw(6) << prn   
       << setw(16)<< xyz[0]
       << setw(16)<< xyz[1]
       << setw(16)<< xyz[2]
       << setprecision(6)
       << setw(16)<< clk*1000000;  // msec

  glog.comment(0,"test",os.str()); os.str("");

  if( gnav )  delete gnav;

  return 0;
}
