
/* ----------------------------------------------------------------------
 * G-Nut - GNSS software development library
 * 
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  This file is part of the G-Nut C++ library.
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation; either version 3 of
  the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, see <http://www.gnu.org/licenses>.

-*/

#include <stdlib.h>
#include <stdio.h>
#include <cstring>

#include "gdata/gdata.h" 
 
using namespace std;

// constructor
// ----------
t_gdata::t_gdata()
  : t_gmonit("gdata"),
    _type(UNKNOWN),
    _group(GRP_UNKNOWN)
{ 
  _log = 0;
}


//  /*
// copy constructor
// ----------
t_gdata::t_gdata( const t_gdata& data )
  : t_gmonit( data )
//    _type(UNKNOWN),
//    _group(GRP_UNKNOWN)
{ 
  _log = data.glog();
  _type = data.id_type();
  _group = data.id_group();
}
//*/

// destructor
// ----------
t_gdata::~t_gdata(){

}

// /*
// assignment operator
// ----------
t_gdata& t_gdata::operator=( const t_gdata& data )
//    _type(UNKNOWN),
//    _group(GRP_UNKNOWN)
{ 
  _log = data.glog();
  _type = data.id_type();
  _group = data.id_group();
  return *this;
}
// */


// set data type
// ----------
int t_gdata::id_type( ID_TYPE t )
{
  unsigned int last = LAST;
  for( unsigned int i = 0; i < last ; ++i ){
    if( t == ID_TYPE(i) ){
      return _type = t;
    }
  }
   
  if( _log ){
    string tmp("Warning: Unknown data id. Reset to 0!\n"); // t !
    _log->comment(1,"gdata",tmp);
  }
   
  _type = t_gdata::UNKNOWN;
  _moni_id = str_type();
  return 0;
}


// set group type
// ----------
int t_gdata::id_group( ID_GROUP g )
{
  unsigned int last = GRP_LAST;
  for( unsigned int i = 0; i < last ; ++i ){
    if( g == ID_GROUP(i) ){
      return _group = g;
    }
  }
  if( _log ){
    string tmp("Warning: Unknown group type. Reset to 0!\n"); // g !
    _log->comment(1,"gdata",tmp);
  }
  _group = t_gdata::GRP_UNKNOWN;
  return 0;
}

// get data type
// ----------
string t_gdata::str_type() const
{
  string type;
  switch( _type ){
   case  UNKNOWN :  type = "UNKOWN";  break;
   case  OBS     :  type = "OBS";     break;
   case  OBSGNSS :  type = "OBSGNSS"; break;
	    
   case  EPH     :  type = "EPH";     break;
// case  EPHALL  :  type = "EPHALL";  break; 
   case  EPHGPS  :  type = "EPHGPS";  break;
   case  EPHGLO  :  type = "EPHGLO";  break;
   case  EPHGAL  :  type = "EPHGAL";  break;
   case  EPHQZSS :  type = "EPHQZSS"; break;
   case  EPHBDS :   type = "EPHBDS";  break;     
   case  EPHSBAS :  type = "EPHSBAS"; break;
   case  EPHPREC :  type = "EPHPREC"; break;
   case  EPHRTCM :  type = "EPHRTCM"; break;
	    
   case  ALLNAV  :  type = "ALLNAV" ; break;
   case  ALLPREC :  type = "ALLPREC"; break;
   case  ALLRTCM :  type = "ALLRTCM"; break;
   case  ALLOBS  :  type = "ALLOBS" ; break;
   case  ALLOBJ  :  type = "ALLOBJ" ; break;
   case  ALLPCV  :  type = "ALLPCV" ; break;
   case  ALLOTL  :  type = "ALLOTL" ; break;
     
   case  POS     :  type = "POS";     break;
   case  POST    :  type = "POST";    break;    
   case  TRPZTD  :  type = "TRPZTD";  break;
	    
   case  PCV     :  type = "PCV";     break;
   case  OTL     :  type = "OTL";     break;
   case  BIAS    :  type = "BIAS";    break;
   case  ERP     :  type = "ERP";     break;
	    
   case  SOL     :  type = "SOL";     break;
   case  LAST    :  type = "UNDEF";   break;
   default       :  type = "UNDEF";
  }
   
return type;
}


// get data type
// ---------- 
string t_gdata::str_group() const
{
  string group;
  switch( _group ){
   case GRP_UNKNOWN : group = "GRP_UNKOWN";  break;
   case GRP_OBSERV  : group = "GRP_OBSERV";  break;
   case GRP_EPHEM   : group = "GRP_EPHEM";   break;
   case GRP_PRODUCT : group = "GRP_PRODUCT"; break;
   case GRP_MODEL   : group = "GRP_MODEL";   break; 
   case GRP_SOLUT   : group = "GRP_SOLUT";   break; 
   case GRP_LAST    : group = "GRP_UNDEF";   break;
   default          : group = "GRP_UNDEF";
  }
return group;
}



// show function
// ----------
string t_gdata::show( int verb, int repeat){

//  boost::mutex::scoped_lock lock(_mutex);
   
//  cout << _moni_id << ":" << endl; 
//  char buff[200 + 1] = "";  
//  sprintf( buff, "%i\n", 2222222222 );
//  sprintf( buff, "%s: %i\n", _moni_id, _data ); // NEFUNGUJE
//  buff[200+1] = '\0';
   
  return NULL;
}

