
#ifndef  GOBSGNSS_H
#define  GOBSGNSS_H

/* ----------------------------------------------------------------------
    (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
       Research Institute of Geodesy, Topography and Cartography
       Ondrejov 244, 251 65, Czech Republic
   
   Purpose: implementation of GNSS observation element
   Version: $Rev:$
       
   2011-09-04 /JD: created
   2012-09-24 /JD: selections of code, phase etc.
   
   Todo: multi-GNSS & various LCs !
         gobj implementation

-*/

#include <string>
#include <vector>
#include <map>

#include "gdata/gdata.h"
#include "gutils/gobs.h"
#include "gutils/gtime.h"
#include "gutils/gsys.h"

// priority tables for choice of available signals (code [m])
const static GOBS code_choise[9][19] = {
   {   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X }, //
   { C1A, C1B, C1C,   X, C1I, C1L, C1M,   X, C1P, C1Q, C1S, C1W, C1X, C1Y, C1Z,  P1,  C1,  CA,  CB }, //  C1
   {   X,   X, C2C, C2D, C2I, C2L, C2M,   X, C2P, C2Q, C2S, C2W, C2X, C2Y,   X,  P2,  C2,  CC,  CD }, //  C2
   {   X,   X,   X,   X, C3I,   X,   X,   X,   X, C3Q,   X,   X, C3X,   X,   X,   X,   X,   X,   X }, //
   {   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X }, //
   {   X,   X,   X,   X, C5I,   X,   X,   X,   X, C5Q,   X,   X, C5X,   X,   X,  P5,  C5,   X,   X }, //  C5
   { C6A, C6B, C6C,   X, C6I,   X,   X,   X,   X, C6Q,   X,   X, C6X,   X, C6Z,   X,  C6,   X,   X }, //  C6
   {   X,   X,   X,   X, C7I,   X,   X,   X,   X, C7Q,   X,   X, C7X,   X,   X,   X,  C7,   X,   X }, //  C7
   {   X,   X,   X,   X, C8I,   X,   X,   X,   X, C8Q,   X,   X, C8X,   X,   X,   X,  C8,   X,   X }  //  C8
};

// priority tables for choice of available signals (phase [full-cycles])
const static GOBS phase_choise[9][19] = {
   {   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X }, //
   { L1A, L1B, L1C,   X, L1I, L1L, L1M, L1N, L1P, L1Q, L1S, L1W, L1X, L1Y, L1Z,   X,  L1,  LA,  LB }, //  L1
   {   X,   X, L2C, L2D, L2I, L2L, L2M, L2N, L2P, L2Q, L2S, L2W, L2X, L2Y,   X,   X,  L2,  LC,  LD }, //  L2
   {   X,   X,   X,   X, L3I,   X,   X,   X,   X, L3Q,   X,   X, L3X,   X,   X,   X,   X,   X,   X }, //
   {   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X }, //
   {   X,   X,   X,   X, L5I,   X,   X,   X,   X, L5Q,   X,   X, L5X,   X,   X,   X,  L5,   X,   X }, //  L5
   { L6A, L6B, L6C,   X, L6I,   X,   X,   X,   X, L6Q,   X,   X, L6X,   X, L6Z,   X,  L6,   X,   X }, //  L6
   {   X,   X,   X,   X, L7I,   X,   X,   X,   X, L7Q,   X,   X, L7X,   X,   X,   X,  L7,   X,   X }, //  L7
   {   X,   X,   X,   X, L8I,   X,   X,   X,   X, L8Q,   X,   X, L8X,   X,   X,   X,  L8,   X,   X }  //  L8
};

/*
// priority tables for choice of available signals (code [m])
const static GOBS code_choise[9][19] = {
   {   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X }, //
   { C1A, C1B, C1C, C1D, C1I, C1L, C1M, C1N, C1P, C1Q, C1S, C1W, C1X, C1Y, C1Z,  P1,  C1,  CA,  CB }, //
   { C2A, C2B, C2C, C2D, C2I, C2L, C2M, C2N, C2P, C2Q, C2S, C2W, C2X, C2Y, C2Z,  P2,  C2,  CC,  CD }, //
   { C3A, C3B, C3C, C3D, C3I, C3L, C3M, C3N, C3P, C3Q, C3S, C3W, C3X, C3Y, C3Z,  P3,  C3,   X,   X }, //
   { C4A, C4B, C4C, C4D, C4I, C4L, C4M, C4N, C4P, C4Q, C4S, C4W, C4X, C4Y, C4Z,  P4,  C4,   X,   X }, //
   { C5A, C5B, C5C, C5D, C5I, C5L, C5M, C5N, C5P, C5Q, C5S, C5W, C5X, C5Y, C5Z,  P5,  C5,   X,   X }, //
   { C6A, C6B, C6C, C6D, C6I, C6L, C6M, C6N, C6P, C6Q, C6S, C6W, C6X, C6Y, C6Z,  P6,  C6,   X,   X }, //
   { C7A, C7B, C7C, C7D, C7I, C7L, C7M, C7N, C7P, C7Q, C7S, C7W, C7X, C7Y, C7Z,  P7,  C7,   X,   X }, //
   { C8A, C8B, C8C, C8D, C8I, C8L, C8M, C8N, C8P, C8Q, C8S, C8W, C8X, C8Y, C8Z,  P8,  C8,   X,   X }, //
};

// priority tables for choice of available signals (phase [full-cycles])
const static GOBS phase_choise[9][19] = {
   {   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X }, //
   { L1A, L1B, L1C, L1D, L1I, L1L, L1M, L1N, L1P, L1Q, L1S, L1W, L1X, L1Y, L1Z,   X,  L1,  LA,  LB }, //
   { L2A, L2B, L2C, L2D, L2I, L2L, L2M, L2N, L2P, L2Q, L2S, L2W, L2X, L2Y, L2Z,   X,  L2,  LC,  LD }, //
   { L3A, L3B, L3C, L3D, L3I, L3L, L3M, L3N, L3P, L3Q, L3S, L3W, L3X, L3Y, L3Z,   X,  L3,   X,   X }, //
   { L4A, L4B, L4C, L4D, L4I, L4L, L4M, L4N, L4P, L4Q, L4S, L4W, L4X, L4Y, L4Z,   X,  L4,   X,   X }, //
   { L5A, L5B, L5C, L5D, L5I, L5L, L5M, L5N, L5P, L5Q, L5S, L5W, L5X, L5Y, L5Z,   X,  L5,   X,   X }, //
   { L6A, L6B, L6C, L6D, L6I, L6L, L6M, L6N, L6P, L6Q, L6S, L6W, L6X, L6Y, L6Z,   X,  L6,   X,   X }, //
   { L7A, L7B, L7C, L7D, L7I, L7L, L7M, L7N, L7P, L7Q, L7S, L7W, L7X, L7Y, L7Z,   X,  L7,   X,   X }, //
   { L8A, L8B, L8C, L8D, L8I, L8L, L8M, L8N, L8P, L8Q, L8S, L8W, L8X, L8Y, L8Z,   X,  L8,   X,   X }, //
};
*/


using namespace std;

#define DEF_CHANNEL 255
const static double NULL_GOBS = 0.0;

class t_gobsgnss : public t_gdata {
   
 public:

   t_gobsgnss();
   t_gobsgnss(const string& site, const string& sat, const t_gtime& t);
   virtual ~t_gobsgnss();
   void addobs(const GOBS& obs,const double& d);  // add a single observation
   void addlli(const GOBS& obs,const int& i);     // add a lost-of-lock indicator
   
   size_t size() const;                           // get number of observations
   double factor( int i );                        // get system-specific factor     for band i   
   double frequency( int i );                     // get system-specific frequency  for band i
   double wavelength( int i );                    // get system-specific wavelength for band i
   double wavelength_L3(int b1 = 1, int b2 = 2);  // get wavelength for iono-free LC
   GSYS gsys();                                   // get GNSS system from satellite IDOC
   GOBS cod_id(const int& band);                  // get  code ID of selected band
   GOBS pha_id(const int& band);                  // get phase ID of selected band

   vector<GOBS> obs();                            // get vector of available observations
   double getobs(const GOBS&   obs);              // get a single observation (DIFFERENT UNITS!)
   double getobs(const string& obs);              // get a single observation (DIFFERENT UNITS!)
   int    getlli(const GOBS&   obs);              // get a lost-of-lock indicator
   int    getlli(const string& obs);              // get a lost-of-lock indicator
   void   nbands(pair<int,int>& nb);              // get number of code/phase available bands
   void   channel(int canal);                    // set channel number for Glonass satellites
   int    channel() const;                        // get channel number for Glonass satellites

// static string freq( FRQ freq );          // get string from FRQ enum
// static FRQ    freq( string freq );       // get FRQ enum from string
// static string obs( GOBS obs );           // get string from GOBS enum
// static GOBS   obs( string obs );         // get GOBS enum from string

   // duplicite with two other methods bellow (OBS-TYPE if pointer given)
   GOBS  code(const int& band,const GOBS& obs=X); // return  code ID of selected band, potentially preffered obs ID
   GOBS phase(const int& band,const GOBS& obs=X); // return phase ID of selected band, potentially preffered obs ID

//   double obs_C(const int& band, GOBS* obs=0);    // get  code observation [m] for band i (1,2,5,6,7,8) + return type
//   double obs_L(const int& band, GOBS* obs=0);    // get phase observation [m] for band i (1,2,5,6,7,8) + return type
   double obs_C(const int& band);                   // get  code observation [m] for band i (1,2,5,6,7,8)
   double obs_L(const int& band);                   // get phase observation [m] for band i (1,2,5,6,7,8)
   double obs_C(const GOBS& gobs);                  // get  code observation [m] for pre-selected type!
   double obs_L(const GOBS& gobs);                  // get phase observation [m] for pre-selected type!
   int mod_L(const double& dL,const GOBS& gobs=X, const int i = 1);  // modify phase observation by adding dL: i = 0 [cycles], i = 1 [m]
    
   double  code_lc(const    int& band1,  // return value of general  code linear combination (c1*O1 + ... )
                   const double& coef1,  // for 2 or 3 bands with given coefficients
	           const    int& band2, 
                   const double& coef2,
	           const    int& band3 = 0, 
                   const double& coef3 = 0);

   double phase_lc(const    int& band1,  // return value of general phase linear combination (c1*O1 + ... ) 
                   const double& coef1,  // for 2 or 3 bands with given coefficients
	           const    int& band2, 
                   const double& coef2,
 	           const    int& band3 = 0, 
                   const double& coef3 = 0);
   
   double frequency_lc(const    int& band1,  // return value of phase linear combination frequency (c1*O1 + ... )
                       const double& coef1,  // for 2 or 3 bands with given coefficients
		       const    int& band2, 
		       const double& coef2,
		       const    int& band3 = 0, 
		       const double& coef3 = 0);
   
   double isf_lc(const int& band1,  // return value of phase linear combination ionospheric scale factor (c1*O1 + ... )
                 const double& coef1,  // for 2 or 3 bands with given coefficients
		 const    int& band2, 
		 const double& coef2,
		 const    int& band3 = 0, 
		 const double& coef3 = 0);   

   double  code_lcf(const int& band1,   // return value of general  code linear combination (c1*f1*O1 + ... / c1*f1 + ) 
                    const int& coef1,   // for 2 or 3 bands with given coefficients
         	    const int& band2, 
                    const int& coef2,
 	            const int& band3 = 0,
                    const int& coef3 = 0);
   
   double phase_lcf(const int& band1,   // return value of general phase linear combination (c1*f1*O1 + ... / c1*f1 + )
                    const int& coef1,   // for 2 or 3 bands with given coefficients
 	            const int& band2, 
                    const int& coef2,   
	            const int& band3 = 0, 
                    const int& coef3 = 0);

   int ionofree_coef(const int& b1, double& c1,   // return coefficients (c1,c2) of the ionosphere-free linear combination
                     const int& b2, double& c2);  // (2 bands)
  
   int geomfree_coef(const int& b1, double& c1,   // return coefficients (c1,c2) of the geometry-free linear combination
                     const int& b2, double& c2);  // (2 bands)
   
   int narrlane_coef(const int& b1, double& c1,   // return coefficients (c1,c2) of the narrow-lane linear combination
                     const int& b2, double& c2);  // (2 bands)

   int widelane_coef(const int& b1, double& c1,   // return coefficients (c1,c2) of the wide-lane linear combination
                     const int& b2, double& c2);  // (2 bands)

   int multpath_coef(const int& bC, double& cC,   // 
                     const int& b1, double& c1,   // return coefficients (c1,c2,c3) of the code multipath linear combination
                     const int& b2, double& c2);  // (2 bands, code & phase)

   double P3(const int& b1=1, const int& b2=2);   // get ionosphere-free combination for code  [m]
   double P4(const int& b1=1, const int& b2=2);   // get geometry-free combination for code    [m]
   
   double L3(const int& b1=1, const int& b2=2);   // get ionosphere-free combination for phase [m]
   double L4(const int& b1=1, const int& b2=2);   // get geometry-free combination for phase   [m]

   double MP(const GOBS& ctype=X,
	     const int& b1=1, const int& b2=2);   // get multipath for C-obs + Li/Lj

   double MW(const int& b1=1, const int& b2=2);   // get Melbourne-Wuebenna combination for phase & code [m]!

   double LWL(const int& b1=1, const int& b2=2);   // get wide-lane combination for phase [m]!
   double LNL(const int& b1=1, const int& b2=2);   // get narrow-lane combination for phase [m]!   
   
   t_gtime epoch() const{ return _epoch; }              // get reference epoch
   string  site()  const{ return _staid; }              // get station id
   string  sat()   const{ return _satid; }              // get satellite id
   string  sys()   const{ return _satid.substr(0,1); }  // get satellite system id

   void   sat(string id) { _satid = id; }          // set satellite id
   void  site(string id) { _staid = id; }          // set site id
   void   epo(t_gtime t) { _epoch = t;  }          // set epoch   
   
//   friend ostream &operator<<(ostream &stream, GOBS  obs);
//   friend istream &operator>>(istream &stream, GOBS &obs);

   t_gobsgnss operator-(t_gobsgnss& obs);
   void clear();
   bool valid();

 protected:
   virtual void _clear();
   virtual bool _valid() const;

   map<GOBS, double>   _gobs;   // maps of observations
   map<GOBS, int>      _glli;   // maps of lost-of-lock identifications
   
   string              _staid;  // station id
   string              _satid;  // satellite id ["G??", "R??", "E??" ...]
   t_gtime             _epoch;  // epoch of the observation
   int                 _channel;// satellite channel number
   bool                _valid_obsL();
   
 private:
   
};

#endif
