
/* ----------------------------------------------------------------------
 * G-Nut - GNSS software development library
 * 
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  This file is part of the G-Nut C++ library.
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation; either version 3 of
  the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, see <http://www.gnu.org/licenses>.

-*/

#include "gutils/gmatrixconv.h"
#include "gutils/gconst.h"
#include "gutils/gtypeconv.h"
#include "gmodels/geop80.h"

using namespace std;

//Constructor
t_geop80::t_geop80()
{
}

//Destructor
t_geop80::~t_geop80()
{
}

// Nutation Matrix (Nutation theory 1980)
// --------------------------------------
Matrix t_geop80::nutMatrix(double mjd)
{
//  boost::mutex::scoped_lock lock(_mutex);
   
   
  const double T  = (mjd-MJD_J2000)/36525.0;

  double ls = 2.0*M_PI*_frac(0.993133+  99.997306*T);
  double D  = 2.0*M_PI*_frac(0.827362+1236.853087*T);
  double F  = 2.0*M_PI*_frac(0.259089+1342.227826*T);
  double N  = 2.0*M_PI*_frac(0.347346-   5.372447*T);

  double dpsi = ( -17.200*sin(N)   - 1.319*sin(2*(F-D+N)) - 0.227*sin(2*(F+N))
                + 0.206*sin(2*N) + 0.143*sin(ls) ) / RHO_SEC;
  double deps = ( + 9.203*cos(N)   + 0.574*cos(2*(F-D+N)) + 0.098*cos(2*(F+N))
                - 0.090*cos(2*N)                 ) / RHO_SEC;

  double eps  = 0.4090928-2.2696E-4*T;

  return  rotX(-eps-deps)*rotZ(-dpsi)*rotX(+eps);  

}

// Precession Matrix (Precession theory 1976)
// ------------------------------------------
Matrix t_geop80::precMatrix (double mjd_1, double mjd_2)
{
//  boost::mutex::scoped_lock lock(_mutex);
   
// Time interval betwen epoch J2000 and start epoch
  const double T  = (mjd_1-MJD_J2000)/36525.0;
   
// Interval over which precession required (JC).
  const double dT = (mjd_2-mjd_1)/36525.0;
  
// Euler angles   
  double zeta  =  ( (2306.2181+(1.39656-0.000139*T)*T)+
                        ((0.30188-0.000344*T)+0.017998*dT)*dT )*dT*SEC2RAD;
  double z     =  zeta + ( (0.79280+0.000411*T)+0.000205*dT)*dT*dT*SEC2RAD;
  double theta =  ( (2004.3109-(0.85330+0.000217*T)*T)-
                        ((0.42665+0.000217*T)+0.041833*dT)*dT )*dT*SEC2RAD;

  return rotZ(-z) * rotY(theta) * rotZ(-zeta);   
}

// Frac part of double
// ------------------------
double t_geop80::_frac (double x)
{ 
   return x-floor(x); 
}
