
#ifndef GSETINP_H
#define GSETINP_H

#define XMLKEY_INP "inputs"

/* ----------------------------------------------------------------------
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  Purpose: implements input setting class
  Version: $ Rev: $

  2012-10-23 /JD: created

-*/

#include <map>
#include <string>
#include <iostream>

#ifdef BMUTEX
  #include <boost/thread/mutex.hpp>
#endif

#include "gio/glog.h"
#include "gutils/gtypeconv.h"
#include "gsetbase.h"

using namespace std;
using namespace pugi;

// the order is important here!
enum IFMT {
     FMT_INP,
     SP3_INP,
     ATX_INP,      // need to be before all OBSERVATIONS to create objects
     RINEXC_INP,
     RINEXN_INP,
     RINEXO_INP,   // need to be after RINEXN (for GLONASS chanels)!
     BNCOBS_INP,   // need to be after RINEXN (for GLONASS chanels)!
     TROSINEX_INP,
     RTCM_INP,
     TSMET_INP,
     BLQ_INP,
     BNCRTCM_INP,  // need to be after RINEXN to fill the corrections !
     GRIB_INP,
     GRID_INP
};


class t_gsetinp : public virtual t_gsetbase
{
 public:
   t_gsetinp();
  ~t_gsetinp();
  
   void check();                                 // settings check
   void help();                                  // settings help

   static IFMT   str2ifmt( const string& s );
   static string ifmt2str( const IFMT&   f );
   
   int input_size(const string& fmt);            // get format input size
   multimap<IFMT,string> inputs_all();           // get format inputs (all in multimap)
   set<string> inputs(const string& fmt);        // get format inputs
   set<string> iformats();                       // get formats
   
 protected:
   set<string> _inputs(const string& fmt);
   set<string> _iformats();

   set<IFMT> _IFMT_supported;                    // vector of supported IFMTs (app-specific)
   
 private:

};

#endif
