
#ifndef GOBS_H
#define GOBS_H
 
/* ----------------------------------------------------------------------
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  Purpose: definition of GNSS observation types
  Version: $ Rev: $

  2012-09-26 /JD: created

-*/

#include <string>
#ifdef BMUTEX
#include <boost/thread/mutex.hpp>
#endif

using namespace std;

// GNSS observations
// -----------------   
enum GOBS {
   
    // psedorange [in meters] - code (RINEX 3.x)
    // 
    C1A = 0,// Galileo A (PRS)     
    C1B,    // Galileo B (I/NAV, OS/CS/SoL)     
    C1C,    // GPS C/A
            // GLONASS C/A
	    // Galileo C (no data)     
	    // SBAS GPS C/A
    C1I,    // BeiDou I
    C1L,    // GPS L1C (L) 
    C1M,    // GPS M 
    C1P,    // GPS P 
            // GLONASS P 
    C1Q,    // BeiDou Q
    C1S,    // GPS L1C (M) 
    C1W,    // GPS Z-tracking (AS on) or similar 
    C1X,    // GPS L1C (M+L) 
            // Galileo B+C
	    // BeiDou I+Q
    C1Y,    // GPS Y 
    C1Z,    // Galileo A+B+C     

    C2C,    // GPS C/A
    C2D,    // GPS L1(C/A)+(P2-P1) pseudorange semi-codeless
    C2I,    // BeiDou E2 I
    C2L,    // GPS L1C (L) 
    C2M,    // GPS M 
    C2P,    // GPS P 
    C2Q,    // BeiDou E2 Q
    C2S,    // GPS L1C (M) 
    C2W,    // GPS Z-tracking (AS on) or similar 
    C2X,    // GPS L1C (M+L) 
            // BeiDou E2 I+Q
    C2Y,    // GPS Y 
     
    C3I,    // GLONASS I
    C3Q,    // GLONASS Q
    C3X,    // GLONASS I+Q

    C5I,    // GPS I 
            // SBAS GPS I     
            // Galileo I (F/NAV, OS)     
    C5Q,    // GPS Q 
            // SBAS GPS Q (no data)     
            // Galileo Q (no data)     
    C5X,    // GPS I+Q 
            // SBAS GPS I+Q     
            // Galileo I+Q     

    C6A,    // Galileo A (PRS)
    C6B,    // Galileo B (C/NAV, OS)     
    C6C,    // Galileo C (no data)     
    C6I,    // BeiDou E6 I
    C6L,    // QZSS L
    C6S,    // QZSS S+L
    C6Q,    // BeiDou E6 Q
    C6X,    // Galileo B+C     
            // BeiDou E6 I+Q
    C6Z,    // Galileo A+B+C     

    C7I,    // Galileo I (I/NAV, OS/CS/SoL)
            // BeiDou E5b I
    C7Q,    // Galileo Q (no data)     
            // BeiDou E5b Q
    C7X,    // Galileo I+Q     
            // BeiDou E5b I+Q
     
    C8I,    // Galileo I     
    C8Q,    // Galileo Q     
    C8X,    // Galileo I+Q

    // phase [in whole cycles] (according to RINEX 3.x)
    // 
    L1A=100,// Galileo A (PRS)     
    L1B,    // Galileo B (I/NAV, OS/CS/SoL)     
    L1C,    // GPS C/A 
            // GLONASS C/A
   	    // Galileo C (no data)     
	    // SBAS GPS C/A 
    L1I,    // BeiDou I
    L1L,    // GPS L1C (L) 
    L1M,    // GPS M 
    L1N,    // GPS codeless
    L1P,    // GPS P 
            // GLONASS P 
    L1Q,    // BeiDou Q
    L1S,    // GPS L1C (M) 
    L1W,    // GPS Z-tracking (AS on) or similar 
    L1X,    // GPS L1C (M+L) 
            // Galileo B+C 
            // BeiDou I+Q
    L1Y,    // GPS Y 
    L1Z,    // Galileo A+B+C     

    L2C,    // GPS C/A 
    L2D,    // GPS L1(C/A)+(P2-P1) phase semi-codeless
    L2I,    // BeiDou E2 I
    L2L,    // GPS L1C (L) 
    L2M,    // GPS M 
    L2N,    // GPS codeless
    L2P,    // GPS P 
    L2Q,    // BeiDou E2 Q
    L2S,    // GPS L1C (M) 
    L2W,    // GPS Z-tracking (AS on) or similar 
    L2X,    // GPS L1C (M+L) 
            // BeiDou E2 I+Q
    L2Y,    // GPS Y 
     
    L3I,    // GLONASS I
    L3Q,    // GLONASS Q
    L3X,    // GLONASS I+Q
     
    L5I,    // GPS I 
            // SBAS GPS I     
            // Galileo I (F/NAV, OS)     
    L5Q,    // GPS Q 
            // SBAS GPS Q (no data)     
            // Galileo Q (no data)     
    L5X,    // GPS I+Q 
            // SBAS GPS I+Q     
            // Galileo I+Q     
     
    L6A,    // Galileo A (PRS)     
    L6B,    // Galileo B (C/NAV, OS)     
    L6C,    // Galileo C (no data)     
    L6I,    // BeiDou E6 I
    L6L,    // QZSS L
    L6S,    // QZSS S+L
    L6Q,    // BeiDou E6 Q
    L6X,    // Galileo B+C     
            // BeiDou E6 I+Q
    L6Z,    // Galileo A+B+C     

    L7I,    // Galileo I (I/NAV, OS/CS/SoL)     
            // BeiDou E5b I
    L7Q,    // Galileo Q (no data)     
            // BeiDou E5b Q
    L7X,    // Galileo I+Q
            // BeiDou E5b I+Q
     
    L8I,    // Galileo I     
    L8Q,    // Galileo Q     
    L8X,    // Galileo I+Q

    // doppler [cycles/sec] (RINEX 3.x)
    //     
    D1A=200,// Galileo A (PRS)     
    D1B,    // Galileo B (I/NAV, OS/CS/SoL)     
    D1C,    // GPS C/A 
            // GLONASS C/A
            // Galileo C (no data)     
            // SBAS GPS C/A 
    D1I,    // BeiDou I
    D1L,    // GPS L1C (L) 
    D1S,    // GPS L1C (M) 
    D1P,    // GPS P 
            // GLONASS P 
    D1Q,    // BeiDou Q
    D1M,    // GPS M 
    D1N,    // GPS codeless
    D1W,    // GPS Z-tracking (AS on) or similar 
    D1X,    // GPS L1C (M+L) 
            // Galileo B+C 
            // BeiDou I+Q
    D1Y,    // GPS Y 
    D1Z,    // Galileo A+B+C     

    D2C,    // GPS C/A 
    D2D,    // GPS L1(C/A)+(P2-P1) dopler semi-codeless
    D2I,    // BeiDou E2 I
    D2L,    // GPS L1C (L) 
    D2M,    // GPS M 
    D2N,    // GPS codeless
    D2P,    // GPS P 
    D2Q,    // BeiDou E2 Q
    D2S,    // GPS L1C (M) 
    D2W,    // GPS Z-tracking (AS on) or similar 
    D2X,    // GPS L1C (M+L) 
            // BeiDou E2 I+Q
    D2Y,    // GPS Y 
     
    D3I,    // GLONASS I
    D3Q,    // GLONASS Q
    D3X,    // GLONASS I+Q
     
    D5I,    // GPS I 
            // SBAS GPS I     
            // Galileo I (F/NAV, OS)     
    D5Q,    // GPS Q 
            // SBAS GPS Q (no data)     
            // Galileo Q (no data)     
    D5X,    // GPS I+Q 
            // SBAS GPS I+Q     
            // Galileo I+Q
	    
    D6A,    // Galileo A (PRS)     
    D6B,    // Galileo B (C/NAV, OS)     
    D6C,    // Galileo C (no data)     
    D6I,    // BeiDou E6 I
    D6L,    // QZSS L
    D6S,    // QZSS S+L
    D6Q,    // BeiDou E6 Q
    D6X,    // Galileo B+C     
            // BeiDou E6 I+Q
    D6Z,    // Galileo A+B+C     

    D7I,    // Galileo I (I/NAV, OS/CS/SoL)     
            // BeiDou E5b I
    D7Q,    // Galileo Q (no data)     
            // BeiDou E5b Q
    D7X,    // Galileo I+Q
            // BeiDou E5b I+Q
     
    D8I,    // Galileo I     
    D8Q,    // Galileo Q     
    D8X,    // Galileo I+Q

    // signal strength [DBHZ] (RINEX 3.x)
    // 
    S1A=300,// Galileo A (PRS)     
    S1B,    // Galileo B (I/NAV, OS/CS/SoL)     
    S1C,    // GPS C/A 
            // GLONASS C/A
            // Galileo C (no data)     
 	    // SBAS GPS C/A 
    S1I,    // BeiDou I
    S1L,    // GPS L1C (L) 
    S1M,    // GPS M 
    S1N,    // GPS codeless
    S1P,    // GPS P 
            // GLONASS P 
    S1Q,    // BeiDou Q
    S1S,    // GPS L1C (M) 
    S1W,    // GPS Z-tracking (AS on) or similar 
    S1X,    // GPS L1C (M+L) 
            // Galileo B+C 
            // BeiDou I+Q
    S1Y,    // GPS Y 
    S1Z,    // Galileo A+B+C     

    S2C,    // GPS C/A 
    S2D,    // GPS L1(C/A)+(P2-P1) signal strength semi-codeless
    S2I,    // BeiDou E2 I
    S2L,    // GPS L1C (L) 
    S2M,    // GPS M 
    S2N,    // GPS codeless
    S2P,    // GPS P 
    S2Q,    // BeiDou E2 Q
    S2S,    // GPS L1C (M) 
    S2W,    // GPS Z-tracking (AS on) or similar 
    S2X,    // GPS L1C (M+L) 
            // BeiDou E2 I+Q
    S2Y,    // GPS Y 
     
    S3I,    // GLONASS I
    S3Q,    // GLONASS Q
    S3X,    // GLONASS I+Q
     
    S5I,    // GPS I 
            // SBAS GPS I     
            // Galileo I (F/NAV, OS)     
    S5Q,    // GPS Q 
            // SBAS GPS Q (no data)     
            // Galileo Q (no data)     
    S5X,    // GPS I+Q 
            // SBAS GPS I+Q     
            // Galileo I+Q     
	    
    S6A,    // Galileo A (PRS)     
    S6B,    // Galileo B (C/NAV, OS)     
    S6C,    // Galileo C (no data)     
    S6I,    // BeiDou E6 I
    S6L,    // QZSS L
    S6S,    // QZSS S+L
    S6Q,    // BeiDou E6 Q
    S6X,    // Galileo B+C     
            // BeiDou E6 I+Q
    S6Z,    // Galileo A+B+C     
     
    S7I,    // Galileo I (I/NAV, OS/CS/SoL)     
            // BeiDou E5b I
    S7Q,    // Galileo Q (no data)     
            // BeiDou E5b Q
    S7X,    // Galileo I+Q     
            // BeiDou E5b I+Q

    S8I,    // Galileo I     
    S8Q,    // Galileo Q     
    S8X,    // Galileo I+Q     

    // special cases: v2.x or unknown tracking modes
    C1=1000,// unknown mode GPS P,Y              Galileo E2-L1-E2 I/Q
    C2,     // unknown mode GPS P,Y                                     BeiDou E2 I/Q
    C5,     // unknown mode                      Galileo E5a I/Q
    C6,     // unknown mode                      Galileo E6  I/Q        BeiDou E6 I/Q
    C7,     // unknown mode                      Galielo E5b I/Q        BeiDou E5b I/Q
    C8,     // unknown mode                      Galileo E5  I/Q
    CA,     // unknown mode GPS C/A  GLONASS SA
    CB,     // unknown mode GPS C1C
    CC,     // unknown mode GPS C2C
    CD,     // unknown mode          GLONASS SA
   
    P1,     // unknown mode GPS P,Y  GLONASS HA
    P2,     // unknown mode GPS P,Y  GLONASS HA
    P5,     // unknown mode GPS P,Y
   
    L1=1100,// unknown mode GPS P,Y              Galileo E2-L1-E2 I/Q
    L2,     // unknown mode GPS P,Y                                     BeiDou E2 I/Q
    L5,     // unknown mode                      Galileo E5a I/Q
    L6,     // unknown mode                      Galileo E6  I/Q        BeiDou E6 I/Q
    L7,     // unknown mode                      Galielo E5b I/Q        BeiDou E5b I/Q
    L8,     // unknown mode                      Galileo E5  I/Q
    LA,     // unknown mode GPS C/A  GLONASS SA
    LB,     // unknown mode GPS C1C
    LC,     // unknown mode GPS C2C
    LD,     // unknown mode          GLONASS SA
     
    D1=1200,// unknown mode GPS P,Y              Galileo E2-L1-E2 I/Q
    D2,     // unknown mode GPS P,Y                                     BeiDou E2 I/Q
    D5,     // unknown mode                      Galileo E5a I/Q
    D6,     // unknown mode                      Galileo E6  I/Q        BeiDou E6 I/Q
    D7,     // unknown mode                      Galielo E5b I/Q        BeiDou E5b I/Q
    D8,     // unknown mode                      Galileo E5  I/Q
    DA,     // unknown mode GPS C/A  GLONASS SA
    DB,     // unknown mode GPS C1C
    DC,     // unknown mode GPS C2C
    DD,     // unknown mode          GLONASS SA
     
    S1=1300,// unknown mode GPS P,Y              Galileo E2-L1-E2 I/Q
    S2,     // unknown mode GPS P,Y                                     BeiDou E2 I/Q
    S5,     // unknown mode                      Galileo E5a I/Q
    S6,     // unknown mode                      Galileo E6  I/Q        BeiDou E6 I/Q
    S7,     // unknown mode                      Galielo E5b I/Q        BeiDou E5b I/Q
    S8,     // unknown mode                      Galileo E5  I/Q
    SA,     // unknown mode GPS C/A  GLONASS SA
    SB,     // unknown mode GPS C1C
    SC,     // unknown mode GPS C2C
    SD,     // unknown mode          GLONASS SA

    X // LAST_GOBS
   };

enum RECTYPE {
   P1P2,    // receiver providing C1, P1, P2
   C1X2,    // cross-correlation
   C1P2     // modern receivers providing C1, P2
};


// class
// ----------
class t_gobs {

 public:
   t_gobs(){};
  ~t_gobs(){};

   static string gobs2str( const GOBS& o );    // get string from GOBS enum
   static GOBS str2gobs( const string& s );    // get GOBS enum from string
   static GOBS pha2snr( const GOBS& o );       // get GOBS enum (pha->snr)
   static int gobs2band( const GOBS& o );      // get band from GOBS enum
   static bool isCode( GOBS o );               // get true for code obs
   static bool isPhase( GOBS o );              // get true for phase obs

 private:
//   boost::mutex   _mutex;

};


#endif // GOBS_H