
/* ----------------------------------------------------------------------
 * G-Nut - GNSS software development library
 * 
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  This file is part of the G-Nut C++ library.
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation; either version 3 of
  the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, see <http://www.gnu.org/licenses>.

-*/

#include <algorithm>
#include <sstream>
#include <cmath>
#include <stdlib.h>
#include <algorithm>

#include "gutils/gtypeconv.h"
#include "gutils/gconst.h"

using namespace std;


// double equivalence according to machine capability
// ----------
bool double_eq(const double& a, const double& b)
{
  if ( abs(a-b)< 1e-30) return true;   // must be editeed with machine epsilon
  else return false;
}


// double to string conversion (width/digits by iomanip can be added !!!)
// ----------
string dbl2str(const double& d, int prec )
{
  ostringstream out;
  out << fixed << setprecision(prec) << " " << setw(0)  << d;
  return out.str();
}


// string to double conversion (avoiding blanks)
// ----------
double str2dbl(const string& s)
{ 
 
  return strtod( s.c_str(), NULL );
   
  // 2x slower alternative using stringstream !
/*
  double i;
  istringstream istr(s);
  istr >> i;
  return i;
*/
}


// string (scientific) to double conversion (avoiding blanks)
// ----------
double strSci2dbl(const string& s)
{   
  double i = 0.0;
  string str(s);
  substitute(str,"d","E");
  substitute(str,"D","E");
  istringstream istr(str);
  istr >> i;
  return i;
}


// integer to string conversion (widtht can be added !!!)
// ----------
string int2str(const int& i)
{
  ostringstream out;
  out << i;
  return out.str();
}


// string to integer conversion (avoiding blanks)
// ----------
int str2int(const string& s)
{
  int i = 0;
  istringstream istr(s);
  istr >> i;
  return i;
}




// return trimmed trailing spaces and changes the content
// ----------
string rtrim(const string& s)
{   
  string str;
  size_t endpos = s.find_last_not_of(" \t");
  if( string::npos != endpos ) str = s.substr( 0, endpos+1 );
  return str;
}

   
// return trimmed string leading spaces
// ----------
string ltrim(const string& s)
{   
  string str;
  size_t startpos = s.find_first_not_of(" \t");
  if( string::npos != startpos ) str = s.substr( startpos );
  return str;
}


// return trimmed leading and tailing spaces
// ----------
string trim(const string& s)
{
  return ltrim( rtrim(s) );
}


// string substitute
// ----------
size_t substitute(string& line, const string& a, const string& b, bool caseSensitive )
{   
   size_t n = 0;
     
   if( caseSensitive ){
     string tmp(line);
     while( (n = line.find(a)) != string::npos ){
       tmp = line.substr(0, n) + b + line.substr( n + a.length() );
       line = tmp;
     }

   }else{
     string lineLC(line);
     string findLC(a);
     transform(lineLC.begin(), lineLC.end(), lineLC.begin(), ::tolower);     
     transform(findLC.begin(), findLC.end(), findLC.begin(), ::tolower);

     while( (n = lineLC.find(findLC)) != string::npos ){
       lineLC = line.substr(0, n) + b + line.substr( n + a.length() );
       line   = lineLC;
     }
   }

   return n+b.length();
}

// Frac part of double
// // ------------------------
 double frac (double x)
 {
    return x-floor(x);
  }
    
